

function Path()
{
	this.nodes = new Array();

	this.addNode = function(x, y)
	{
		var n = new Node(x, y);
		this.nodes.push(n);
		return n;
	}

	this.connectNodes = function(n0, n1)
	{
		n0.connectTo(n1);
		n1.connectTo(n0);
	}

	this.update = function()
	{
		for (var i = 0; i < this.nodes.length; i++)
			this.nodes[i].update();
	}

	this.showPath = function(nsrc, ndest)
	{
		if (nsrc == ndest)
			return null;

		if (!this.calculateRoute(nsrc, ndest))
			return null;

		// Count how many nodes are in the route.
		var n = ndest;
		var count = 1;
		while (n.from != nsrc)
		{
			n = n.from;
			count += 4;
		}
		
		n = ndest;
		while (n.from != nsrc)
		{
			n.flashIn(count);
			n = n.from;
			count -= 4;
		}
		
		n.flashIn(count);
	}

	this.findNext = function(nsrc, ndest)
	{
		if (nsrc == ndest)
			return null;

		if (!this.calculateRoute(nsrc, ndest))
			return null;

		var n = ndest;
		while (n.from != nsrc)
			n = n.from;

		return n;
	}

	this.calculateRoute = function(nsrc, ndest)
	{
		if (nsrc == ndest)
			return null;

		var current = [nsrc];
		nsrc.from = null;
		
		var keepgoing = true;

		while (current.indexOf(ndest) == -1 && keepgoing)
		{
			keepgoing = false;

			for (var i = current.length - 1; i >= 0; i--)
			{
				for (var j = 0; j < current[i].connections.length; j++)
				{
					// If this node is blocked by a barrier or enemy then skip it.
					if (current[i].barrier != null || current[i].enemy != null)
						continue;

					// If we've already been to this node then continue.
					if (current.indexOf(current[i].connections[j]) != -1)
						continue;

					current[i].connections[j].from = current[i];
					current.push(current[i].connections[j]);
					keepgoing = true;
				}
			}
		}
		
		return (current.indexOf(ndest) != -1);
	}


	this.findNearestNode = function(pos)
	{
		var bestnode = null;
		var bestdist = Number.MAX_VALUE;

		for (var i = 0; i < this.nodes.length; i++)
		{
			var newdist = Global.vectorToDist(this.nodes[i].pos, pos);
			if (newdist < 50 && newdist < bestdist)
			{
				bestdist = newdist;
				bestnode = this.nodes[i];
			}
		}
		return bestnode;
	}

	this.render = function()
	{
		for (var i = 0; i < this.nodes.length; i++)
			this.nodes[i].render();
	}
}
